@@ -8,19 +8,19 @@ describe Cocoon do
 
   subject {TestClass.new}
 
-  it { should respond_to(:link_to_add_association) }
-  it { should respond_to(:link_to_remove_association) }
+  it { is_expected.to respond_to(:link_to_add_association) }
+  it { is_expected.to respond_to(:link_to_remove_association) }
 
   before(:each) do
     @tester = TestClass.new
     @post = Post.new
-    @form_obj = double(:object => @post, :object_name => @post.class.name)
   end
 
 
   context "link_to_add_association" do
     before(:each) do
-      @tester.stub(:render_association).and_return('form<tag>')
+      @form_obj = double(:object => @post, :object_name => @post.class.name)
+      allow(@tester).to receive(:render_association).and_return('form<tag>')
     end
 
     context "without a block" do
@@ -33,6 +33,28 @@ describe Cocoon do
         it_behaves_like "a correctly rendered add link", {}
       end
 
+      context 'and no name given' do
+        context 'custom translation exists' do
+          before do
+            I18n.backend.store_translations(:en, :cocoon => { :comments => { :add => 'Add comment' } })
+
+            @html = @tester.link_to_add_association(@form_obj, :comments)
+          end
+
+          it_behaves_like "a correctly rendered add link", { text: 'Add comment' }
+        end
+
+        context 'uses default translation' do
+          before do
+            I18n.backend.store_translations(:en, :cocoon => { :defaults => { :add => 'Add' } })
+
+            @html = @tester.link_to_add_association(@form_obj, :comments)
+          end
+
+          it_behaves_like "a correctly rendered add link", { text: 'Add' }
+        end
+      end
+
       context "and given html options to pass them to link_to" do
         before do
           @html = @tester.link_to_add_association('add something', @form_obj, :comments, {:class => 'something silly'})
@@ -43,8 +65,8 @@ describe Cocoon do
 
       context "and explicitly specifying the wanted partial" do
         before do
-          @tester.unstub(:render_association)
-          @tester.should_receive(:render_association).with(anything(), anything(), anything(), "f", anything(), "shared/partial").and_return('partiallll')
+          allow(@tester).to receive(:render_association).and_call_original
+          expect(@tester).to receive(:render_association).with(anything(), anything(), anything(), "f", anything(), "shared/partial").and_return('partiallll')
           @html = @tester.link_to_add_association('add something', @form_obj, :comments, :partial => "shared/partial")
         end
 
@@ -52,15 +74,15 @@ describe Cocoon do
       end
 
       it "gives an opportunity to wrap/decorate created objects" do
-        @tester.unstub(:render_association)
-        @tester.should_receive(:render_association).with(anything(), anything(), kind_of(CommentDecorator), "f", anything(), anything()).and_return('partiallll')
+        allow(@tester).to receive(:render_association).and_call_original
+        expect(@tester).to receive(:render_association).with(anything(), anything(), kind_of(CommentDecorator), "f", anything(), anything()).and_return('partiallll')
         @tester.link_to_add_association('add something', @form_obj, :comments, :wrap_object => Proc.new {|comment| CommentDecorator.new(comment) })
       end
 
       context "force non association create" do
         context "default case: create object on association" do
           before do
-            @tester.should_receive(:create_object).with(anything, :comments , false)
+            expect(@tester).to receive(:create_object).with(anything, :comments , false)
             @html = @tester.link_to_add_association('add something', @form_obj, :comments)
           end
 
@@ -69,7 +91,7 @@ describe Cocoon do
 
         context "and explicitly specifying false is the same as default" do
           before do
-            @tester.should_receive(:create_object).with(anything, :comments , false)
+            expect(@tester).to receive(:create_object).with(anything, :comments , false)
             @html = @tester.link_to_add_association('add something', @form_obj, :comments, :force_non_association_create => false)
           end
           it_behaves_like "a correctly rendered add link", {}
@@ -77,7 +99,7 @@ describe Cocoon do
 
         context "specifying true will not create objects on association but using the conditions" do
           before do
-            @tester.should_receive(:create_object).with(anything, :comments , true)
+            expect(@tester).to receive(:create_object).with(anything, :comments , true)
             @html = @tester.link_to_add_association('add something', @form_obj, :comments, :force_non_association_create => true)
           end
           it_behaves_like "a correctly rendered add link", {}
@@ -127,8 +149,8 @@ describe Cocoon do
 
       context "and explicitly specifying the wanted partial" do
         before do
-          @tester.unstub(:render_association)
-          @tester.should_receive(:render_association).with(anything(), anything(), anything(), "f", anything(), "shared/partial").and_return('partiallll')
+          allow(@tester).to receive(:render_association).and_call_original
+          expect(@tester).to receive(:render_association).with(anything(), anything(), anything(), "f", anything(), "shared/partial").and_return('partiallll')
           @html = @tester.link_to_add_association( @form_obj, :comments, :class => 'floppy disk', :partial => "shared/partial") do
             "some long name"
           end
@@ -157,13 +179,13 @@ describe Cocoon do
     end
 
     it "tttt" do
-      @post.class.reflect_on_association(:people).klass.new.should be_a(Person)
+      expect(@post.class.reflect_on_association(:people).klass.new).to be_a(Person)
     end
 
     context "with extra render-options for rendering the child relation" do
       context "uses the correct plural" do
         before do
-          @tester.should_receive(:render_association).with(:people, @form_obj, anything, "f", {:wrapper => 'inline'}, nil)
+          expect(@tester).to receive(:render_association).with(:people, @form_obj, anything, "f", {:wrapper => 'inline'}, nil)
           @html = @tester.link_to_add_association('add something', @form_obj, :people, :render_options => {:wrapper => 'inline'})
         end
         it_behaves_like "a correctly rendered add link", {association: 'person', associations: 'people' }
@@ -173,18 +195,22 @@ describe Cocoon do
     context "passing locals to the partial" do
       context "when given: passes the locals to the partials" do
         before do
-          @tester.unstub(:render_association)
-          @form_obj.should_receive(:fields_for) { | association, new_object, options_hash, &block| block.call }
-          @tester.should_receive(:render).with("person_fields", {:f=>nil, :dynamic=>true, :alfred=>"Judoka"}).and_return ("partiallll")
-          @html = @tester.link_to_add_association('add something', @form_obj, :people, :render_options => {:wrapper => 'inline', :locals => {:alfred => 'Judoka'}})
+          allow(@tester).to receive(:render_association).and_call_original
+          expect(@form_obj).to receive(:fields_for) { | association, new_object, options_hash, &block| block.call }
+          expect(@tester).to receive(:render).with("person_fields", {:f=>nil, :dynamic=>true, :alfred=>"Judoka"}).and_return ("partiallll")
+          @render_options = {:wrapper => 'inline', :locals => {:alfred => 'Judoka'}}
+          @html = @tester.link_to_add_association('add something', @form_obj, :people, :render_options => @render_options)
         end
         it_behaves_like "a correctly rendered add link", {template: 'partiallll', association: 'person', associations: 'people' }
+        it "does not afffect render-options" do
+          expect(@render_options[:locals]).to eq({alfred: 'Judoka'})
+        end
       end
       context "if no locals are given it still works" do
         before do
-          @tester.unstub(:render_association)
-          @form_obj.should_receive(:fields_for) { | association, new_object, options_hash, &block| block.call }
-          @tester.should_receive(:render).with("person_fields", {:f=>nil, :dynamic=>true}).and_return ("partiallll")
+          allow(@tester).to receive(:render_association).and_call_original
+          expect(@form_obj).to receive(:fields_for) { | association, new_object, options_hash, &block| block.call }
+          expect(@tester).to receive(:render).with("person_fields", {:f=>nil, :dynamic=>true}).and_return ("partiallll")
           @html = @tester.link_to_add_association('add something', @form_obj, :people, :render_options => {:wrapper => 'inline'})
         end
         it_behaves_like "a correctly rendered add link", {template: 'partiallll', association: 'person', associations: 'people' }
@@ -196,9 +222,9 @@ describe Cocoon do
     context "overruling the form parameter name" do
       context "when given a form_name it passes it correctly to the partials" do
         before do
-          @tester.unstub(:render_association)
-          @form_obj.should_receive(:fields_for) { | association, new_object, options_hash, &block| block.call }
-          @tester.should_receive(:render).with("person_fields", {:people_form => nil, :dynamic=>true}).and_return ("partiallll")
+          allow(@tester).to receive(:render_association).and_call_original
+          expect(@form_obj).to receive(:fields_for) { | association, new_object, options_hash, &block| block.call }
+          expect(@tester).to receive(:render).with("person_fields", {:people_form => nil, :dynamic=>true}).and_return ("partiallll")
           @html = @tester.link_to_add_association('add something', @form_obj, :people, :form_name => 'people_form')
         end
         it_behaves_like "a correctly rendered add link", {template: 'partiallll', association: 'person', associations: 'people' }
@@ -208,13 +234,14 @@ describe Cocoon do
 
     context "when using formtastic" do
       before(:each) do
-        @tester.unstub(:render_association)
-        @form_obj.stub(:semantic_fields_for).and_return('form<tagzzz>')
+        allow(@tester).to receive(:render_association).and_call_original
+        allow(@form_obj).to receive(:semantic_fields_for).and_return('form<tagzzz>')
       end
       context "calls semantic_fields_for and not fields_for" do
         before do
-          @form_obj.should_receive(:semantic_fields_for)
-          @form_obj.should_receive(:fields_for).never
+          allow(@form_obj).to receive_message_chain(:class, :ancestors) { ['Formtastic::FormBuilder'] }
+          expect(@form_obj).to receive(:semantic_fields_for)
+          expect(@form_obj).to receive(:fields_for).never
           @html = @tester.link_to_add_association('add something', @form_obj, :people)
         end
         it_behaves_like "a correctly rendered add link", {template: 'form<tagzzz>', association: 'person', associations: 'people' }
@@ -222,16 +249,17 @@ describe Cocoon do
     end
     context "when using simple_form" do
       before(:each) do
-        @tester.unstub(:render_association)
-        @form_obj.stub(:simple_fields_for).and_return('form<tagxxx>')
+        allow(@tester).to receive(:render_association).and_call_original
+        allow(@form_obj).to receive(:simple_fields_for).and_return('form<tagxxx>')
       end
       it "responds_to :simple_fields_for" do
-        @form_obj.should respond_to(:simple_fields_for)
+        expect(@form_obj).to respond_to(:simple_fields_for)
       end
       context "calls simple_fields_for and not fields_for" do
         before do
-          @form_obj.should_receive(:simple_fields_for)
-          @form_obj.should_receive(:fields_for).never
+          allow(@form_obj).to receive_message_chain(:class, :ancestors) { ['SimpleForm::FormBuilder'] }
+          expect(@form_obj).to receive(:simple_fields_for)
+          expect(@form_obj).to receive(:fields_for).never
           @html = @tester.link_to_add_association('add something', @form_obj, :people)
         end
         it_behaves_like "a correctly rendered add link", {template: 'form<tagxxx>', association: 'person', associations: 'people' }
@@ -248,6 +276,9 @@ describe Cocoon do
   end
 
   context "link_to_remove_association" do
+    before do
+      @form_obj = @tester.send(:instantiate_builder, @post.class.name, @post, {})
+    end
     context "without a block" do
       context "accepts a name" do
         before do
@@ -257,14 +288,36 @@ describe Cocoon do
         it "is rendered inside a input element" do
           doc = Nokogiri::HTML(@html)
           removed = doc.at('input')
-          removed.attribute('id').value.should == "Post__destroy"
-          removed.attribute('name').value.should == "Post[_destroy]"
-          removed.attribute('value').value.should == "false"
+          expect(removed.attribute('id').value).to eq("Post__destroy")
+          expect(removed.attribute('name').value).to eq("Post[_destroy]")
+          expect(removed.attribute('value').value).to eq("false")
         end
 
         it_behaves_like "a correctly rendered remove link", {}
       end
 
+      context 'no name given' do
+        context 'custom translation exists' do
+          before do
+            I18n.backend.store_translations(:en, :cocoon => { :posts => { :remove => 'Remove post' } })
+
+            @html = @tester.link_to_remove_association(@form_obj)
+          end
+
+          it_behaves_like "a correctly rendered remove link", { text: 'Remove post' }
+        end
+
+        context 'uses default translation' do
+          before do
+            I18n.backend.store_translations(:en, :cocoon => { :defaults => { :remove => 'Remove' } })
+
+            @html = @tester.link_to_remove_association(@form_obj)
+          end
+
+          it_behaves_like "a correctly rendered remove link", { text: 'Remove' }
+        end
+      end
+
       context "accepts html options and pass them to link_to" do
         before do
           @html = @tester.link_to_remove_association('remove something', @form_obj, {:class => 'add_some_class', :'data-something' => 'bla'})
@@ -280,16 +333,16 @@ describe Cocoon do
       before do
         @post_marked_for_destruction = Post.new
         @post_marked_for_destruction.mark_for_destruction
-        @form_obj_destroyed = double(:object => @post_marked_for_destruction, :object_name => @post_marked_for_destruction.class.name)
+        @form_obj_destroyed = @tester.send(:instantiate_builder, @post_marked_for_destruction.class.name, @post_marked_for_destruction, {})
         @html = @tester.link_to_remove_association('remove something', @form_obj_destroyed)
       end
 
       it "is rendered inside a input element" do
         doc = Nokogiri::HTML(@html)
         removed = doc.at('input')
-        removed.attribute('id').value.should == "Post__destroy"
-        removed.attribute('name').value.should == "Post[_destroy]"
-        removed.attribute('value').value.should == "true"
+        expect(removed.attribute('id').value).to eq("Post__destroy")
+        expect(removed.attribute('name').value).to eq("Post[_destroy]")
+        expect(removed.attribute('value').value).to eq("true")
       end
 
       it_behaves_like "a correctly rendered remove link", {class: 'remove_fields dynamic destroyed'}
@@ -336,24 +389,24 @@ describe Cocoon do
 
   context "create_object" do
     it "creates correct association with conditions" do
-      @tester.should_not_receive(:create_object_with_conditions)
+      expect(@tester).not_to receive(:create_object_with_conditions)
       # in rails4 we cannot create an associated object when the object has not been saved before
       # I submitted a bug for this: https://github.com/rails/rails/issues/11376
       if Rails.rails4?
         @post = Post.create(title: 'Testing')
-        @form_obj = double(:object => @post, :object_name => @post.class.name)
       end
-      result = @tester.create_object(@form_obj, :admin_comments)
-      result.author.should == "Admin"
-      @form_obj.object.admin_comments.should be_empty
+      form_obj = double(:object => @post, :object_name => @post.class.name)
+      result = @tester.create_object(form_obj, :admin_comments)
+      expect(result.author).to eq("Admin")
+      expect(form_obj.object.admin_comments).to be_empty
     end
 
     it "creates correct association for belongs_to associations" do
       comment  = Comment.new
       form_obj = double(:object => Comment.new)
       result   = @tester.create_object(form_obj, :post)
-      result.should be_a Post
-      comment.post.should be_nil
+      expect(result).to be_a Post
+      expect(comment.post).to be_nil
     end
 
     it "raises an error if cannot reflect on association" do
@@ -362,30 +415,31 @@ describe Cocoon do
 
     it "creates an association if object responds to 'build_association' as singular" do
       object = Comment.new
-      object.should_receive(:build_custom_item).and_return 'custom'
-      @tester.create_object(double(:object => object), :custom_item).should == 'custom'
+      expect(object).to receive(:build_custom_item).and_return 'custom'
+      expect(@tester.create_object(double(:object => object), :custom_item)).to eq('custom')
     end
 
     it "creates an association if object responds to 'build_association' as plural" do
       object = Comment.new
-      object.should_receive(:build_custom_item).and_return 'custom'
-      @tester.create_object(double(:object => object), :custom_items).should == 'custom'
+      expect(object).to receive(:build_custom_item).and_return 'custom'
+      expect(@tester.create_object(double(:object => object), :custom_items)).to eq('custom')
     end
 
     it "can create using only conditions not the association" do
-      @tester.should_receive(:create_object_with_conditions).and_return('flappie')
-      @tester.create_object(@form_obj, :comments, true).should == 'flappie'
+      expect(@tester).to receive(:create_object_with_conditions).and_return('flappie')
+      form_obj = @tester.send(:instantiate_builder, @post.class.name, @post, {})
+      expect(@tester.create_object(form_obj, :comments, true)).to eq('flappie')
     end
   end
 
   context "get_partial_path" do
     it "generates the default partial name if no partial given" do
       result = @tester.get_partial_path(nil, :admin_comments)
-      result.should == "admin_comment_fields"
+      expect(result).to eq("admin_comment_fields")
     end
     it "uses the given partial name" do
       result = @tester.get_partial_path("comment_fields", :admin_comments)
-      result.should == "comment_fields"
+      expect(result).to eq("comment_fields")
     end
   end
 
