@@ -4,7 +4,7 @@ class CreatePosts < ActiveRecord::Migration
       t.string :title
       t.text :body
 
-      t.timestamps
+      t.timestamps null: false
     end
   end
 
